@@ -87,6 +87,13 @@ class Gencontrol(Base):
                         'Unable to disable {desc} in release build ({env} set)'
                         .format(desc=desc, env=env))
 
+        # In case installer packages are disabled through
+        # $DEBIAN_KERNEL_DISABLE_INSTALLER, update the config (that
+        # gets written to config.defines.dump) so that they will
+        # also be disabled in gencontrol_signed.py.
+        if self.disable_installer:
+            self.config.setdefault(('packages',), {})['installer'] = False
+
     def _setup_makeflags(self, names, makeflags, data):
         for src, dst, optional in names:
             if src in data or not optional:
@@ -272,14 +279,6 @@ class Gencontrol(Base):
 
     def do_arch_packages(self, packages, makefile, arch, vars, makeflags,
                          extra):
-        if self.version.linux_modifier is None:
-            try:
-                abiname_part = '-%s' % self.config['abi', arch]['abiname']
-            except KeyError:
-                abiname_part = self.abiname_part
-            makeflags['ABINAME'] = vars['abiname'] = \
-                self.abiname_version + abiname_part
-
         if not self.disable_signed:
             build_signed = self.config.merge('build', arch) \
                                       .get('signed-code', False)
@@ -647,8 +646,6 @@ class Gencontrol(Base):
                                     arch, featureset, flavour))
         makeflags['KCONFIG'] = ' '.join(kconfig)
         makeflags['KCONFIG_OPTIONS'] = ''
-        if build_signed:
-            makeflags['KCONFIG_OPTIONS'] += ' -o SECURITY_LOCKDOWN_LSM=y -o MODULE_SIG=y'
         # Add "salt" to fix #872263
         makeflags['KCONFIG_OPTIONS'] += \
             ' -o "BUILD_SALT=\\"%(abiname)s%(localversion)s\\""' % vars
@@ -682,16 +679,58 @@ class Gencontrol(Base):
                 'image-dbg', vars,
                 'linux-image-%(abiname)s%(localversion)s-dbg' % vars)
 
+    # Generate a unique kernel ABI name suffix following the old
+    # (pre-trixie) format
+    def _generate_abiname_part(self):
+        # Start with 35 for 5.10.237-1 (the last manually set ABI
+        # version) and add 1 for every non-backport changelog entry
+        # after that.  Also count backport entries since the last
+        # non-backport, in case we need further disambiguation.
+        abiname_major = 0
+        backport_count = 0
+        target_release = None
+        for entry in self.changelog:
+            if '~' in entry.version.revision:
+                if target_release is None:
+                    target_release = entry.version.revision.split('u')[0]
+                if abiname_major == 0:
+                    # Only increase for backport entries for the target release
+                    if target_release in entry.version.revision:
+                        backport_count += 1
+            elif str(entry.version) == '5.10.237-1':
+                abiname_major += 35
+                break
+            else:
+                abiname_major += 1
+
+        # Select an appropriate prefix for the target release
+        for release_name, release_prefix in [
+                ('UNRELEASED', ''),
+                ('bullseye',   ''),
+                ('buster',     '0.deb10.'),
+                ('stretch',    '0.deb9.'),
+        ]:
+            if self.changelog[0].distribution.startswith(release_name):
+                break
+        else:
+            assert False, f'cannot generate ABI name for {self.changelog[0].distribution}'
+
+        # Paste that all together
+        result = f'-{release_prefix}{abiname_major}'
+        if backport_count > 1:
+            result += f'.{backport_count - 1}'
+        return result
+
     def process_changelog(self):
         version = self.version = self.changelog[0].version
-        if self.version.linux_modifier is not None:
-            self.abiname_part = ''
-        else:
+        try:
             self.abiname_part = '-%s' % self.config['abi', ]['abiname']
+        except KeyError:
+            self.abiname_part = self._generate_abiname_part()
         # We need to keep at least three version components to avoid
         # userland breakage (e.g. #742226, #745984).
         self.abiname_version = re.sub(r'^(\d+\.\d+)(?=-|$)', r'\1.0',
-                                      self.version.linux_upstream)
+                                      self.version.linux_version)
         self.vars = {
             'upstreamversion': self.version.linux_upstream,
             'version': self.version.linux_version,
